@@ -1,7 +1,7 @@
 // Paraselene generated this source file.
 package #1;
 
-import #1.base.*;
+import #1.#x.*;
 import paraselene.supervisor.*;
 import java.io.*;
 
@@ -9,7 +9,7 @@ import java.io.*;
 /**
  * JSON非同期通信。
  */
-public class Json extends #1.base.Json {
+public class Json extends #1.#x.Json {
 	private static final long serialVersionUID = 1L;
 
 	/**
