@@ -1,8 +1,8 @@
 // Paraselene generated this source file.
-package #1.logic#2;
+package #1.#z#2;
 
 import #1.*;
-import #1.base.*;
+import #1.#x.*;
 import paraselene.*;
 import paraselene.supervisor.*;
 import paraselene.tag.*;
@@ -14,7 +14,7 @@ import paraselene.css.*;
 /**
  * #3
  */
-public class #4 extends #1.view#2.#4 {
+public class #4 extends #1.#y#2.#4 {
 	private static final long serialVersionUID = 1L;
 
 	/**
