@@ -1,5 +1,5 @@
 // Paraselene generated this source file.
-package #1.view#2;
+package #1.#y#2;
 
 import paraselene.*;
 import paraselene.tag.*;
@@ -36,10 +36,10 @@ e.printStackTrace();
 }
 /**
  * ページIDの取得。
- * @return ページID。#1.base.PageType.#6。
+ * @return ページID。#1.#x.PageType.#6。
  */
 public paraselene.supervisor.PageID getID() {
-return #1.base.PageType.#6;
+return #1.#x.PageType.#6;
 }
 #7
 }
