@@ -46,7 +46,9 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 			buf = buf.append( "pf.returnPage(page);\n" );
 			buf = buf.append( "page=new " );
 			buf = buf.append( Param.PACKAGE.get() );
-			buf = buf.append( ".base.Css();\n" );
+			buf = buf.append( "." );
+			buf = buf.append( Param.PACKAGE_BASE.get() );
+			buf = buf.append( ".Css();\n" );
 			buf = buf.append( "pf.addDefine(PARASELENE_CSS,page.getClass());\n");
 			buf = buf.append( "map.put(PARASELENE_CSS.id,PARASELENE_CSS);\n" );
 			buf = buf.append( "pf.returnPage(page);\n" );
@@ -68,6 +70,12 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 			return buf.toString();
 		case 4:
 			return Param.OUT_SOURCE_PATH.get();
+		case 33:
+			return Param.PACKAGE_BASE.get();
+		case 34:
+			return Param.PACKAGE_VIEW.get();
+		case 35:
+			return Param.PACKAGE_LOGIC.get();
 		}
 		return null;
 	}
@@ -124,7 +132,7 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 			if ( output[i] == null )	continue;
 			StringBuilder    out_path = new StringBuilder( Make.SRC_PATH );
 			out_path = out_path.append( File.separator );
-			out_path = out_path.append( "base" );
+			out_path = out_path.append( Param.PACKAGE_BASE.get() );
 			out_path = out_path.append( File.separator );
 			out_path = out_path.append( name[i] );
 			String  p_name = out_path.toString();
@@ -158,6 +166,12 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 		getTitleTitle().setValueString( Param.PACKAGE.get() );
 		getRootTd().setValueString( "<" + Param.OUT_PATH.get() + ">" );
 		getSourceTd().setValueString( " <" + Param.OUT_SOURCE_PATH.get() + ">" );
+		getBaseSpan().setValueString( Param.PACKAGE_BASE.get() );
+		getViewSpan().setValueString( Param.PACKAGE_VIEW.get() );
+		Tag[]	logic = getLogicTags();
+		for ( int i = 0; i < logic.length; i++ ) {
+			logic[i].setValueString( Param.PACKAGE_LOGIC.get() );
+		}
 
 		Table	table = getClassListTable();
 		Line	line = table.getLineReplica( 1 );
@@ -174,7 +188,7 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 				page.getTitle(),
 				page.getClassName(),
 				page.getEnumName( false ),
-				TransactionSequencer.makeURI( null, null, page.getID() )
+				TransactionSequencer.makeURI( null, null, page.getID(), 0, 0, false )
 			);
 			String	note = page.getNote();
 			if ( note == null )	note = "";
