@@ -95,6 +95,18 @@ public enum Param {
 		"ソースファイルのルートパッケージ名。\n" +
 		"パッケージはルートパッケージ名を起点に階層化されます。",
 		true, true  ),
+	PACKAGE_BASE( "-package_base",
+		"baseパッケージ名。\n" +
+		"baseパッケージ名を指定パッケージ名に変更します。",
+		false, true, "base" ),
+	PACKAGE_VIEW( "-package_view",
+		"viewパッケージ名。\n" +
+		"viewパッケージ名を指定パッケージ名に変更します。",
+		false, true, "view" ),
+	PACKAGE_LOGIC( "-package_logic",
+		"logicパッケージ名。\n" +
+		"logicパッケージ名を指定パッケージ名に変更します。",
+		false, true, "logic" ),
 	OTHER( "-other",
 		"画像などの静的コンテンツのルートパス。\n" +
 		HTML_ROOT.key + " とこのパスが一致しているとみなし、\n" +
@@ -274,6 +286,23 @@ public enum Param {
 		}
 	}
 
+	static void package_check( Param p ) {
+		String[]	pkg = p.get().split( "\\." );
+		for ( int i = 0; i < pkg.length; i++ ) {
+			if ( pkg[i].isEmpty() || JavaDefine.isDefine( pkg[i], true ) ) {
+				System.err.println( p.key + " に使用できない文字が含まれています" );
+				System.exit( 1 );
+			}
+		}
+	}
+
+	static void package_check( Param p1, Param p2 ) {
+		if ( p1.get().equals( p2.get() ) ) {
+			System.err.println( p1.key + " と " + p2.key + " が同一です" );
+			System.exit( 1 );
+		}
+	}
+
 	static void loadParam() {
 		if ( param_cnt < 1 )	help();
 
@@ -291,13 +320,14 @@ public enum Param {
 			System.exit( 1 );
 		}
 		Make.SRC_PATH = OUT_SOURCE_PATH.get();
-		String[]	pkg = PACKAGE.get().split( "\\." );
-		for ( int i = 0; i < pkg.length; i++ ) {
-			if ( JavaDefine.isDefine( pkg[i], true ) ) {
-				System.err.println( PACKAGE.key + " に使用できない文字が含まれています" );
-				System.exit( 1 );
-			}
-		}
+		package_check( PACKAGE );
+		package_check( PACKAGE_BASE );
+		package_check( PACKAGE_VIEW );
+		package_check( PACKAGE_LOGIC );
+		package_check( PACKAGE_BASE, PACKAGE_VIEW );
+		package_check( PACKAGE_VIEW, PACKAGE_LOGIC );
+		package_check( PACKAGE_LOGIC, PACKAGE_BASE );
+		PrePage.setDir( PACKAGE_VIEW.get(), PACKAGE_LOGIC.get() );
 		if ( !LineSeparator.setUse( SOURCE_LINE.get() ) ) {
 			System.err.println( SOURCE_LINE.key + " の指定が不正です" );
 			System.exit( 1 );
