@@ -18,6 +18,7 @@ class Child extends Thread {
 	volatile Forward forward = null;
 	volatile Page param_page = null;
 	volatile Throwable th = null;
+	volatile long	thread_id = -1;
 
 	Child() {
 		setDaemon( true );
@@ -25,6 +26,7 @@ class Child extends Thread {
 	}
 
 	public void run() {
+		thread_id = Thread.currentThread().getId();
 		try {
 			synchronized( this ) {
 				wait();
@@ -67,6 +69,60 @@ public class SandBox extends Thread {
 	private static volatile long	wait_ms = 3 * 60 * 1000;
 	private static SandBox sand_box = null;
 
+	private static Child getMyself() {
+		long	my_id = Thread.currentThread().getId();
+		synchronized( running ) {
+			int	cnt = running.size();
+			for ( int i = 0; i < cnt; i++ ) {
+				Child	child = running.get( i );
+				if ( child.thread_id == my_id )	return child;
+			}
+		}
+		return null;
+	}
+
+	/**
+	 * 実行中ページの取得。
+	 * 各ページの、input または output が呼ばれている時、自身のページを
+	 * いつでも参照できます。<br>
+	 * input や output 経由で呼ばれているのであれば、どのクラスメソッドからでも
+	 * 参照可能です。
+	 * @return 実行中のページ。
+	 */
+	public static Page getCurrentPage() {
+		Child	child = getMyself();
+		if ( child == null )	return null;
+		return child.main_page;
+	}
+
+	/**
+	 * リクエストパラメータの取得。
+	 * 各ページの、input または output が呼ばれている時、現在のリクエストパラメータを
+	 * いつでも参照できます。<br>
+	 * input や output 経由で呼ばれているのであれば、どのクラスメソッドからでも
+	 * 参照可能です。
+	 * @return 現在のリクエストパラメータ。
+	 */
+	public static RequestParameter getCurrentRequestParameter() {
+		Child	child = getMyself();
+		if ( child == null )	return null;
+		return child.request;
+	}
+
+	/**
+	 * Input処理中であるか。
+	 * 各ページの、input または output が呼ばれている時、現在どちらを
+	 * 処理しているか返します。<br>
+	 * input や output 経由で呼ばれているのであれば、どのクラスメソッドからでも
+	 * 参照可能です。
+	 * @return true:input、false:input以外 output や firstOutput。
+	 */
+	public static boolean isCurrentInput() {
+		Child	child = getMyself();
+		if ( child == null )	return false;
+		return child.call_input;
+	}
+
 	private SandBox(){}
 
 	/**
