@@ -9,17 +9,30 @@ package paraselene.tag;
 import java.util.*;
 import java.io.*;
 import java.net.*;
+import java.math.*;
 import paraselene.*;
+import paraselene.tag.form.*;
 import paraselene.supervisor.*;
 
 /**
  * HTML汎用タグ。
- * スレッドセーフです。
  */
 public class Tag extends HTMLPart {
 	private static final Page	null_page = new NullPage(){
 		public Forward input( RequestParameter req, Forward fw ) throws PageException {return null;}
 	};
+
+	private static HashMap<String, String>	name_ok_map = new HashMap<String, String>();
+	static {
+		String[] name_ok = {
+			"applet", "area", "embed", "frame", "frameset", "iframe", "ilayer",
+			"img", "layer", "map", "meta", "object", "param", 
+		};
+		for ( int  i = 0; i < name_ok.length; i++ ) {
+			name_ok_map.put( name_ok[i], name_ok[i] );
+		}
+	}
+
 	private String name;
 	protected HashMap<String,Attribute>	attr = new HashMap<String,Attribute>();
 	private ArrayList<HTMLPart>		doc = new ArrayList<HTMLPart>();
@@ -27,6 +40,7 @@ public class Tag extends HTMLPart {
 	private boolean nest_f = false;
 	private boolean visible_f = true;
 	private boolean dirty_f = true;
+	private short[]	location_no = new short[]{ 0 };
 
 	private static String	VALUE = "value";
 
@@ -59,7 +73,7 @@ public class Tag extends HTMLPart {
 		return getAssignedTag();
 	}
 
-	private synchronized Tag[] getOverUnder() {
+	private Tag[] getOverUnder() {
 		Tag[]	ret = new Tag[]{ null, null };
 		int	cnt = getAssignedTag().tag_only.size();
 		for ( int i = 0; i < cnt; i++ ) {
@@ -153,10 +167,8 @@ public class Tag extends HTMLPart {
 	private void makeID() {
 		if ( getAttribute( "id" ) != null )	return;
 		long	now = 0;
-		synchronized( VALUE ){
-			now = tag_id;
-			tag_id++;
-		}
+		now = tag_id;
+		tag_id++;
 		long	time = (now << 56) | (new Date().getTime() & 0x00ffffffffffffffL);
 		StringBuilder	buf = new StringBuilder( PARASELENE_ID );
 		buf = buf.append( Long.toString( time, Character.MAX_RADIX ) );
@@ -198,10 +210,8 @@ public class Tag extends HTMLPart {
 	 */
 	public Attribute[] getAllAttribute() {
 		ArrayList<Attribute>	list = new ArrayList<Attribute>();
-		synchronized( this ) {
-			for ( String k : attr.keySet() ) {
-				list.add( attr.get( k ) );
-			}
+		for ( String k : attr.keySet() ) {
+			list.add( attr.get( k ) );
 		}
 		return list.toArray( new Attribute[0] );
 	}
@@ -212,10 +222,11 @@ public class Tag extends HTMLPart {
 	 * addHTMLPart()する時に配列化にされる場合があります。
 	 * @return 複製。
 	 */
-	public synchronized HTMLPart getReplica() {
+	public HTMLPart getReplica() {
 		Tag	tag = newReplica();
 		tag.visible_f = this.visible_f;
 		for ( String k : attr.keySet() ) {
+			if ( "id".equals( k ) )	continue;
 			tag.setAttribute( attr.get( k ).getReplica() );
 		}
 		int	size = doc.size();
@@ -256,9 +267,7 @@ public class Tag extends HTMLPart {
 				}
 			}
 		}
-		synchronized( this ) {
-			attr.put( at.getName(), at );
-		}
+		attr.put( at.getName(), at );
 		at.setAssignedTag( this );
 		if ( "name".equals( at_name ) ) {
 			Tag	out = getOuterTag();
@@ -280,7 +289,7 @@ public class Tag extends HTMLPart {
 	 * 属性の削除。
 	 * @param at_name 属性名。
 	 */
-	public synchronized void removeAttribute( String at_name ) {
+	public void removeAttribute( String at_name ) {
 		setModify();
 		attr.remove( at_name.toLowerCase( Locale.ENGLISH ) );
 	}
@@ -290,7 +299,7 @@ public class Tag extends HTMLPart {
 	 * @param at_name 属性名。
 	 * @return 属性。
 	 */
-	public synchronized Attribute getAttribute( String at_name ) {
+	public Attribute getAttribute( String at_name ) {
 		return attr.get( at_name.toLowerCase( Locale.ENGLISH ) );
 	}
 
@@ -305,6 +314,50 @@ public class Tag extends HTMLPart {
 		return attr.getString();
 	}
 
+	/**
+	 * 属性値を数値にして取得。数値化に失敗するとnullを返す。<br>
+	 * 属性値がNumberValueの場合にのみ成功します。
+	 * @param at_name 属性名。
+	 * @return 属性値の数値表現。
+	 */
+	public Integer getAttributeToInteger( String at_name ) {
+		NumberValue	num = getAttributeToNumberValue( at_name );
+		if ( num == null )	return null;
+		return num.toInteger();
+	}
+
+	/**
+	 * 属性値を数値にして取得。数値化に失敗するとnullを返す。<br>
+	 * 属性値がNumberValueの場合にのみ成功します。
+	 * @param at_name 属性名。
+	 * @return 属性値の数値表現。
+	 */
+	public Double getAttributeToDouble( String at_name ) {
+		NumberValue	num = getAttributeToNumberValue( at_name );
+		if ( num == null )	return null;
+		return num.toDouble();
+	}
+
+	/**
+	 * 属性値を数値にして取得。数値化に失敗するとnullを返す。<br>
+	 * 属性値がNumberValueの場合にのみ成功します。
+	 * @param at_name 属性名。
+	 * @return 属性値の数値表現。
+	 */
+	public BigDecimal getAttributeToBigDecimal( String at_name ) {
+		NumberValue	num = getAttributeToNumberValue( at_name );
+		if ( num == null )	return null;
+		return num.toBigDecimal();
+	}
+
+	private NumberValue getAttributeToNumberValue( String at_name ) {
+		Attribute	attr = getAttribute( at_name );
+		if ( attr == null )	return null;
+		AttributeValuable	av = attr.get();
+		if ( !(av instanceof NumberValue) )	return null;
+		return (NumberValue)av;
+	}
+
 	/**
 	 * HTML文書の追加。name属性値が衝突すると、それを変更します。
 	 * 末尾に追加されます。
@@ -314,12 +367,94 @@ public class Tag extends HTMLPart {
 		addHTMLPart( doc.size(), d );
 	}
 
+	/**
+	 * タグのソート。タグをHTML先頭からの登場順にソートします。
+	 * @param array ソート対象。この配列インスタンス自体がソートされます。
+	 * @return array を返します。
+	 */
+	public static Tag[] sort( Tag[] array ) {
+		if ( array == null )	return null;
+		if ( array.length <= 1 )	return array;
+		int	h = sortDiv( array.length );
+		while( true ) {
+			if ( h == 9 || h == 10 )	h = 11;
+			boolean	done = false;
+			for ( int i = 0; (i + h) < array.length; i++ ) {
+				boolean	swap_f = false;
+				if ( array[i + h].location_no.length < array[i].location_no.length ) {
+					swap_f = true;
+				}
+				else if ( array[i + h].location_no.length == array[i].location_no.length ) {
+					for ( int j = 0; j < array[i].location_no.length; j++ ) {
+						if ( array[i + h].location_no[j] < array[i].location_no[j] ) {
+							swap_f = true;
+							break;
+						}
+						if ( array[i + h].location_no[j] > array[i].location_no[j] ) {
+							break;
+						}
+					}
+				}
+				if ( swap_f ) {
+					Tag	tmp = array[i + h];
+					array[i + h] = array[i];
+					array[i] = tmp;
+					done = true;
+				}
+			}
+			if ( h <= 1 && !done ) return array;
+			h = sortDiv( h );
+		}
+	}
+
+	private static final BigInteger BI13 = new BigInteger( "13" );
+
+	private static int sortDiv( int org ) {
+		BigInteger	bd = new BigInteger( Integer.toString( org ) );
+		bd = bd.multiply( BigInteger.TEN );
+		bd = bd.divide( BI13 );
+		int	ret = bd.intValue();
+		if ( ret < 1 )	ret = 1;
+		return ret;
+	}
+
+	private void setLocation() {
+		Tag[]	list = getTagArray();
+		short	last_no = -1;
+		for ( int i = 0; i < list.length; i++ ) {
+			if ( list[i].location_no.length == (this.location_no.length + 1) ) {
+				boolean	flag = true;
+				for ( int j = 0; j < this.location_no.length; j++ ) {
+					if ( list[i].location_no[j] != this.location_no[j] ) {
+						flag = false;
+						break;
+					}
+				}
+				if ( flag && list[i].location_no[this.location_no.length] > last_no ) {
+					last_no = list[i].location_no[this.location_no.length];
+					continue;
+				}
+			}
+			list[i].location_no = new short[this.location_no.length + 1];
+			for ( int j = 0; j < this.location_no.length; j++ ) {
+				list[i].location_no[j] = this.location_no[j];
+			}
+			last_no++;
+			if ( last_no < 0 ) {
+				Option.debug( "tag overflow!" );
+				last_no = 0;
+			}
+			list[i].location_no[this.location_no.length] = last_no;
+			list[i].setLocation();
+		}
+	}
+
 	/**
 	 * HTML文書の追加。name属性値が衝突すると、それを変更します。
 	 * @param idx 追加位置。
 	 * @param d 文書。
 	 */
-	public synchronized void addHTMLPart( int idx, HTMLPart d ) {
+	public void addHTMLPart( int idx, HTMLPart d ) {
 		setModify();
 		doc.add( idx, d );
 		if ( d instanceof Tag ) {
@@ -336,6 +471,7 @@ public class Tag extends HTMLPart {
 			}
 			tag_only.add( idx, tag );
 			if ( tag.getAttribute( "name" ) != null )	makeID();
+			setLocation();
 		}
 		d.setAssignedTag( this );
 	}
@@ -365,14 +501,15 @@ public class Tag extends HTMLPart {
 
 	/**
 	 * タグのインクルード。<br>
-	 * タグで挟まれた部分全てを取り込む。そのタグ自体は取り込まない。
+	 * タグで挟まれた部分全てを取り込む。そのタグ自体は取り込まない。<br>
+	 * getReplica()したものを取り込みます。
 	 * @param tag 取り込むタグ。
 	 */
-	public synchronized void include( Tag tag ) {
+	public void include( Tag tag ) {
 		if ( tag == null )	return;
 		int	cnt = tag.getHTMLPartCount();
 		for ( int i = 0; i < cnt; i++ ) {
-			addHTMLPart( tag.getHTMLPart( i ) );
+			addHTMLPart( tag.getHTMLPart( i ).getReplica() );
 		}
 	}
 
@@ -381,7 +518,7 @@ public class Tag extends HTMLPart {
 	 * bodyタグの内側部分全てを取り込む。
 	 * @param page 取り込むページ。
 	 */
-	public synchronized void include( Page page ) {
+	public void include( Page page ) {
 		Tag	tag = page.getFirstTagByType( "body" );
 		if ( tag == null )	return;
 		include( tag );
@@ -391,7 +528,7 @@ public class Tag extends HTMLPart {
 	 * HTML文書の取得。
 	 * @param idx 取得位置。
 	 */
-	public synchronized HTMLPart getHTMLPart( int idx ) {
+	public HTMLPart getHTMLPart( int idx ) {
 		return doc.get( idx );
 	}
 
@@ -399,7 +536,7 @@ public class Tag extends HTMLPart {
 	 * HTML文書件数の取得。
 	 * @return 件数。
 	 */
-	public synchronized int getHTMLPartCount() {
+	public int getHTMLPartCount() {
 		return doc.size();
 	}
 
@@ -407,7 +544,7 @@ public class Tag extends HTMLPart {
 	 * HTML文書の削除。
 	 * @param idx 削除位置。
 	 */
-	public synchronized void removeHTMLPart( int idx ) {
+	public void removeHTMLPart( int idx ) {
 		HTMLPart	p = doc.remove( idx );
 		if ( p == null )	return;
 		removeHTMLPart( p );
@@ -417,7 +554,7 @@ public class Tag extends HTMLPart {
 	 * HTML文書の削除。最初に登場した指定インスタンスを削除します。
 	 * @param part 削除対象。
 	 */
-	public synchronized void removeHTMLPart( HTMLPart part ) {
+	public void removeHTMLPart( HTMLPart part ) {
 		if ( part == null )	return;
 		setModify();
 		if ( part instanceof Tag ) {
@@ -432,7 +569,7 @@ public class Tag extends HTMLPart {
 	/**
 	 * HTML文書の全削除。
 	 */
-	public synchronized void removeHTMLPart() {
+	public void removeHTMLPart() {
 		int	cnt = doc.size();
 		for ( cnt--; cnt >= 0; cnt-- ) {
 			removeHTMLPart( cnt );
@@ -444,7 +581,7 @@ public class Tag extends HTMLPart {
 	 * removeHTMLPartと異なり、除去対象タグの内部のデータが残存します。
 	 * @param tag 除去対象。
 	 */
-	public synchronized void exclude( Tag tag ) {
+	public  void exclude( Tag tag ) {
 		int	idx = indexOf( tag );
 		removeHTMLPart( tag );
 		int	cnt = tag.getHTMLPartCount();
@@ -457,7 +594,7 @@ public class Tag extends HTMLPart {
 	 * 位置を返す。最初に登場した指定インスタンスの位置を返します。
 	 * @param part 検出対象。無ければ-1を返す。
 	 */
-	public synchronized int indexOf( HTMLPart part ) {
+	public int indexOf( HTMLPart part ) {
 		if ( part == null ) return -1;
 		return doc.indexOf( part );
 	}
@@ -466,12 +603,18 @@ public class Tag extends HTMLPart {
 	 * 開始タグの文字列。
 	 * @return 開始タグ。
 	 */
-	protected synchronized String getFirstTag() {
+	protected String getFirstTag() {
 		boolean	xml = false;
 		xml = getAssignedPage().isXML();
 		StringBuilder	buf = new StringBuilder( "<" );
-		buf = buf.append( getName() );
+		boolean	name_f = ( this instanceof Anchor || this instanceof Form || this instanceof Control );
+		String	myname = getName();
+		if ( !name_f ) {
+			name_f = ( name_ok_map.get( myname ) != null );
+		}
+		buf = buf.append( myname );
 		for ( String k : attr.keySet() ) {
+			if ( "name".equals( k ) && !name_f )	continue;
 			buf = buf.append( " " );
 			buf = buf.append( attr.get( k ).toHtmlString( xml ) );
 		}
@@ -523,7 +666,7 @@ public class Tag extends HTMLPart {
 	 * 文書の出力。
 	 * @retrun 文書の文字列化。
 	 */
-	private synchronized String getHTMLParts( StringMode mode ) {
+	private String getHTMLParts( StringMode mode ) {
 
 		int	cnt = getHTMLPartCount();
 		StringBuilder	buf = new StringBuilder();
@@ -591,13 +734,11 @@ public class Tag extends HTMLPart {
 	 */
 	public String getValueString() {
 		if  ( nest_f ) {
-			synchronized( this ) {
-				int	cnt = getHTMLPartCount();
-				for ( int i = 0; i < cnt; i++ ) {
-					HTMLPart	pt = getHTMLPart( i );
-					if ( pt instanceof Text ) {
-						return ((Text)pt).toString( StringMode.PLAIN );
-					}
+			int	cnt = getHTMLPartCount();
+			for ( int i = 0; i < cnt; i++ ) {
+				HTMLPart	pt = getHTMLPart( i );
+				if ( pt instanceof Text ) {
+					return ((Text)pt).toString( StringMode.PLAIN );
 				}
 			}
 			return null;
@@ -629,17 +770,15 @@ public class Tag extends HTMLPart {
 	 */
 	public void setValueString( String v ) {
 		if ( nest_f ) {
-			synchronized( this ) {
-				int	cnt = getHTMLPartCount();
-				for ( int i = 0; i < cnt; i++ ) {
-					HTMLPart	pt = getHTMLPart( i );
-					if ( pt instanceof Text ) {
-						((Text)pt).setText( v, StringMode.PLAIN );
-						return;
-					}
+			int	cnt = getHTMLPartCount();
+			for ( int i = 0; i < cnt; i++ ) {
+				HTMLPart	pt = getHTMLPart( i );
+				if ( pt instanceof Text ) {
+					((Text)pt).setText( v, StringMode.PLAIN );
+					return;
 				}
-				addHTMLPart( new Text( v ) );
 			}
+			addHTMLPart( new Text( v ) );
 			return;
 		}
 		setValueAttribute( v );
@@ -660,7 +799,7 @@ public class Tag extends HTMLPart {
 	 * 再帰的に全てのタグを探すわけではありません。
 	 * @return タグ。無ければ0個の配列を返す。
 	 */
-	public synchronized Tag[] getTagArray() {
+	public Tag[] getTagArray() {
 		return tag_only.toArray( new Tag[0] );
 	}
 
@@ -669,7 +808,7 @@ public class Tag extends HTMLPart {
 	 * 指定クラスと同一、または指定クラスから派生したクラスを取り出し
 	 * 配列にして返します。再帰的に探します。
 	 * @param cls 探すクラス。Tagの派生クラスを指定します。
-	 * @return タグ。無ければ0個の配列を返す。
+	 * @return タグ。無ければ0個の配列を返す。登場順にソートされてます。
 	 * ただし、clsがnullなら、nullを返す。
 	 */
 	public Tag[] getAllTagByClass( Class<?> cls ) {
@@ -685,13 +824,13 @@ public class Tag extends HTMLPart {
 				list.add( crt[i] );
 			}
 		}
-		return list.toArray( new Tag[0] );
+		return sort( list.toArray( new Tag[0] ) );
 	}
 
 	/**
 	 * タグの取得。AやTABLEのような、タグの種別で探します。再帰的に探します。
 	 * @param name タグの種別。
-	 * @return タグの配列。無ければ0個の配列を返す。
+	 * @return タグの配列。無ければ0個の配列を返す。登場順にソートされてます。
 	 * ただし、nameがnullなら、nullを返す。
 	 */
 	public Tag[] getAllTagByType( String name ) {
@@ -708,7 +847,7 @@ public class Tag extends HTMLPart {
 				list.add( crt[i] );
 			}
 		}
-		return list.toArray( new Tag[0] );
+		return sort( list.toArray( new Tag[0] ) );
 	}
 
 	/**
@@ -748,17 +887,15 @@ public class Tag extends HTMLPart {
 		if ( target instanceof Tag ) {
 			return ((Tag)target).getOuterTag();
 		}
-		synchronized( this ) {
-			int	cnt = doc.size();
-			for ( int i = 0; i < cnt; i++ ) {
-				HTMLPart	part = doc.get( i );
-				if ( part == target )	return this;
-				if ( part instanceof Tag ) {
-					Tag	tag = ((Tag)part).getDirectInnerTag( target );
-					if ( tag != null )	return tag;
-				}
-			}	
-		}
+		int	cnt = doc.size();
+		for ( int i = 0; i < cnt; i++ ) {
+			HTMLPart	part = doc.get( i );
+			if ( part == target )	return this;
+			if ( part instanceof Tag ) {
+				Tag	tag = ((Tag)part).getDirectInnerTag( target );
+				if ( tag != null )	return tag;
+			}
+		}	
 		return null;
 	}
 
@@ -785,7 +922,7 @@ public class Tag extends HTMLPart {
 	/**
 	 * BRタグを改行コードに置換する。
 	 */
-	public synchronized void replaceBR() {
+	public void replaceBR() {
 		int	cnt = getHTMLPartCount();
 		for ( int i = 0; i < cnt; i++ ) {
 			HTMLPart	p = getHTMLPart( i );
@@ -816,5 +953,72 @@ public class Tag extends HTMLPart {
 		if ( p == null )	p = null_page;
 		embed_page = p;
 	}
+
+	private ArrayList<HTMLPart> getInnerHTMLPart( ArrayList<HTMLPart> list, ArrayList<HTMLPart> d ) {
+		int	tgt_cnt = d.size();
+		if ( tgt_cnt == 0 )	return list;
+		int	cnt = getHTMLPartCount();
+		for ( int i = 0; i < cnt; i++ ) {
+			HTMLPart	data = getHTMLPart( i );
+			for ( int j = tgt_cnt; j >= 0; j-- ) {
+				if ( data == d.get( j ) ) {
+					list.add( data );
+					d.remove( data );
+					tgt_cnt--;
+					if ( tgt_cnt == 0 )	return list;
+				}
+			}
+			if ( data instanceof Tag ) {
+				Tag	tag = (Tag)data;
+				list = tag.getInnerHTMLPart( list, d );
+			}
+		}
+		return list;
+	}
+
+	/**
+	 * 内包物の検索。
+	 * 引数で指定されたインスタンス群のうち、this が内包しているもののみを抽出して
+	 * 返します。<br>
+	 * 直接内包ではなく、再帰的にネストされたタグを検索します。
+	 * @param d 検索対象の集合。
+	 * @return 存在した要素の集合。無ければ空の配列を返します。
+	 */
+	public HTMLPart[] getInnerHTMLPart( HTMLPart[] d ) {
+		if ( d == null )	return new HTMLPart[0];
+		ArrayList<HTMLPart>	list = new ArrayList<HTMLPart>();
+		ArrayList<HTMLPart>	tgt = new ArrayList<HTMLPart>();
+		for ( int i = 0; i < d.length; i++ ) {
+			tgt.add( d[i] );
+		}
+		return getInnerHTMLPart( list, tgt ).toArray( new HTMLPart[0] );
+	}
+
+	/**
+	 * 内包物の検索。
+	 * 引数で指定されたインスタンス群のうち、this が内包しているもので
+	 * 最初に見つかったものを返します。<br>
+	 * 直接内包ではなく、再帰的にネストされたタグを検索します。
+	 * @param d 検索対象の集合。
+	 * @return 存在した要素。無ければnullを返します。
+	 */
+	public HTMLPart getFirstInnerHTMLPart( HTMLPart[] d ) {
+		if ( d == null )	return null;
+		if ( d.length == 0 )	return null;
+		int	cnt = getHTMLPartCount();
+		for ( int i = 0; i < cnt; i++ ) {
+			HTMLPart	data = getHTMLPart( i );
+			for ( int j = 0; j < d.length; j++ ) {
+				if ( data == d[j] ) return data;
+			}
+			if ( data instanceof Tag ) {
+				Tag	tag = (Tag)data;
+				HTMLPart	ret = tag.getFirstInnerHTMLPart( d );
+				if ( ret != null )	return ret;
+			}
+		}
+		return null;
+	}
+
 }
 
